/* Copyright 2006 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: GameApp.cpp,v 1.2 2006-04-30 22:11:53 tonic Exp $
 * $Revision: 1.2 $
 */

#include "main.h"


GameApp::GameApp() :
    mStateInitStatus(TRUE),
    mGraphics(NULL),
    mBestScore(0)
{
}


GameApp::~GameApp()
{
    deinit();
    delete mGraphics;
}


BOOL GameApp::init()
{
    BOOL result = App::init();

    if (!result)
        return result;

    FILE *fp = fopen("score.dat", "rb");
    if (fp != NULL)
    {
        fread(&mBestScore, sizeof(INT32), 1, fp);
        fclose(fp);
    }

    //mGraphics = new UIGraphicsOpenGLSDLCFL(gCFL);
    mGraphics = new UIGraphicsOpenGLSDL();
    if (mGraphics == NULL)
        return FALSE;

    UIFontOpenGLSDL *font;

    font = new UIFontOpenGLSDL(mGraphics, DATA_PREFIX "font_treb_small.png");
    if (!font || !font->init())
        return FALSE;
    font->setCharacterSpacing(-3);
    mFonts[FONT_SMALL] = font;

    font = new UIFontOpenGLSDL(mGraphics, DATA_PREFIX "font_treb_medium.png");
    if (!font || !font->init())
        return FALSE;
    mFonts[FONT_MEDIUM] = font;

    font = new UIFontOpenGLSDL(mGraphics, DATA_PREFIX "font_treb_big.png");
    if (!font || !font->init())
        return FALSE;
    mFonts[FONT_BIG] = font;

    font = new UIFontOpenGLSDL(mGraphics, DATA_PREFIX "font_block.png");
    if (!font || !font->init())
        return FALSE;
    mFonts[FONT_BLOCK] = font;

    initAndAddState(STATE_TITLE, new TitleState);
    initAndAddState(STATE_GAME_RUN, new GameState(mGraphics));
    if (!mStateInitStatus)
        return FALSE;

    // TODO: FIXME
    setState(STATE_TITLE);
    //setState(STATE_GAME_RUN);

    return TRUE;
}


void GameApp::deinit()
{
    deinitAndDeleteStates();
    deleteFonts();
}


void GameApp::initAndAddState(STATE_TYPE stateId, State *newState)
{
    if (!mStateInitStatus)
        return;

    if (!newState->init())
    {
        mStateInitStatus = FALSE;
        delete newState;
        return;
    }
    else
        mStates[stateId] = newState;
}


void GameApp::deinitAndDeleteStates()
{
    StateMap::iterator it = mStates.begin();
    StateMap::iterator itend = mStates.end();
    for (; it != itend; ++it)
    {
        State *state = (*it).second;
        state->deinit();
        delete state;
    }
    mStates.clear();
}


void GameApp::deleteFonts()
{
    INT a;
    for (a = 0; a < FONT_COUNT; ++a)
        delete mFonts[a];
}


void GameApp::setBestScore(INT32 score)
{
    mBestScore = score;
    FILE *fp = fopen("score.dat", "wb");
    if (fp != NULL)
    {
        fwrite(&mBestScore, sizeof(INT32), 1, fp);
        fclose(fp);
    }
}


char * GameApp::getCaption()
{
    return CAPTION_STR;
}


State * GameApp::getStateHandler(STATE_TYPE state)
{
    State *res = mStates[state];
    assert(res != NULL);
    return res;
}
