/* Copyright 2006 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: TitleState.cpp,v 1.3 2006-04-30 22:11:53 tonic Exp $
 * $Revision: 1.3 $
 */

#include <math.h>
#include "main.h"


TitleState::TitleState() :
    mPlayButton(NULL),
    mExitButton(NULL),
    mScoreLabel(NULL),
    mButtonUpImage(NULL),
    mButtonDownImage(NULL),
    mBackgroundImage(NULL)
{
}


TitleState::~TitleState()
{
    deinit();
}


BOOL TitleState::init()
{
    BOOL success = TRUE;

    GameApp *ga = GameApp::getInstance();
    INT scrW = ga->getScreenWidth(), scrH = ga->getScreenHeight();

    UIGraphics *graphics = ga->getGraphics();
    mUI.setGraphics(graphics);

    UIFont *medium = ga->getMediumFont();
    UIFont *small = ga->getSmallFont();
    UIFont *big = ga->getBigFont();

    mButtonUpImage = graphics->loadImage(DATA_PREFIX "button_up.png");
    mButtonDownImage = graphics->loadImage(DATA_PREFIX "button_down.png");
    if (!mButtonDownImage || !mButtonDownImage)
        success = FALSE;

    if (success)
    {
        mBackgroundImage = graphics->loadImage(DATA_PREFIX "menu_background.jpg", UIImage::FILTER_LINEAR);
        if (!mBackgroundImage)
            success = FALSE;
    }

    const UINT buttonWidth = mButtonUpImage->getWidth();
    const UINT buttonHeight = mButtonUpImage->getHeight();
    const UINT gap = 30;

    mPlayButton = new UIButton(scrW / 3 - buttonWidth / 2, scrH / 4,
                               mButtonUpImage, mButtonDownImage, NULL,
                               medium, "Play");
    mPlayButton->setColor(0.9f, 1, 0.9f, 1);
    mPlayButton->addEventListener(this);
    mUI.addUIComponent(mPlayButton);
    if (!mPlayButton)
        success = FALSE;

    mExitButton = new UIButton(scrW / 3 - buttonWidth / 2, scrH / 4 + buttonHeight * 3 / 2,
                               mButtonUpImage, mButtonDownImage, NULL,
                               medium, "Exit");
    mExitButton->setColor(1, 0.9f, 0.9f, 1);
    mExitButton->addEventListener(this);
    mUI.addUIComponent(mExitButton);
    if (!mExitButton)
        success = FALSE;

    const UINT lgap = big->getFontHeight();
    UILabel *label;

    FLOAT y = scrH * 2 / 3;
    label = new UILabel(scrW / 2, y, NULL, big,
                        "L o n e   W a r r i o r", UIFont::ALIGN_HORIZ_CENTER);
    mUI.addUIComponent(label);
    if (!label)
        success = FALSE;

    y += lgap;
    label = new UILabel(scrW / 2, y, NULL, small,
                        "Version " VERSION_STR, UIFont::ALIGN_HORIZ_CENTER);
    label->setItemColor(0.7f, 0.7f, 0.7f, 1);
    mUI.addUIComponent(label);
    if (!label)
        success = FALSE;

    y += gap;
    mScoreLabel = new UILabel(scrW / 2, y, NULL, medium,
                              "", UIFont::ALIGN_HORIZ_CENTER);
    mScoreLabel->setItemColor(0.75f, 0.9f, 1, 1);
    mUI.addUIComponent(mScoreLabel);
    if (!mScoreLabel)
        success = FALSE;

    label = new UILabel(scrW / 2, scrH - 1, NULL, small,
                        "Copyright 2006 Jetro Lauha - http://jet.ro",
                        UIFont::ALIGN_HORIZ_CENTER | UIFont::ALIGN_BOTTOM);
    label->setItemColor(0.75f, 0.75f, 0.75f, 1);
    mUI.addUIComponent(label);
    if (!label)
        success = FALSE;

    mUI.activateAllComponents();

    return success;
}


void TitleState::onActivate(UINT32 /*time*/)
{
    int chn = playSoundEffect(SOUND_BASE, BASE_SND_CHANNEL);
    setSoundChannelVolume(BASE_SND_CHANNEL, 0);
    setSoundChannelFrequency(BASE_SND_CHANNEL, 20000);

    setMusic(TRUE, MUSIC_BASE);

    GameApp *ga = GameApp::getInstance();
    INT32 bestScore = ga->getBestScore();
    if (bestScore > 0)
    {
        char tmp[512];
        sprintf(tmp, "Best score: %d", bestScore);
        mScoreLabel->setText(tmp);
    }
}


void TitleState::deinit()
{
    mUI.deleteAllUIComponents();

    delete mButtonUpImage;
    mButtonUpImage = NULL;
    delete mButtonDownImage;
    mButtonDownImage = NULL;
    delete mBackgroundImage;
    mBackgroundImage = NULL;
}


BOOL TitleState::update(UINT32 time)
{
    mTime = time;

    return TRUE;
}


void TitleState::render(SDL_Surface *screen)
{
    //glClearColor(0, 0, 0, 1);
    //glClear(GL_COLOR_BUFFER_BIT);

    GameApp *app = (GameApp *)App::getInstance();

    UIGraphics *g = GameApp::getInstance()->getGraphics();
    g->beginRender();
    //mBackgroundImage->setTargetSize((FLOAT)WIDTH, (FLOAT)HEIGHT);
    //mBackgroundImage->draw(UIImage::NORMAL, UIImage::DEFAULT_FLAGS, 0, 0);
    FLOAT minScale = (FLOAT)WIDTH / mBackgroundImage->getWidth();
    mBackgroundImage->setScale((FLOAT)sin(mTime * 0.0001) * 0.5f + 0.5f + minScale);
    mBackgroundImage->draw(UIImage::NORMAL,
        UIImage::TRANSFORM | UIImage::SCALE | UIImage::CENTER,
        WIDTH / 2, HEIGHT / 2);
    g->endRender();

    mUI.render();
}


void TitleState::onKeyDown(const SDLKey &key)
{
    if (key == SDLK_SPACE || key == SDLK_KP_ENTER || key == SDLK_RETURN)
        gameStart();
    if (key == SDLK_ESCAPE)
        GameApp::getInstance()->setAlive(FALSE);
}


void TitleState::onMouseDown(const SDL_MouseButtonEvent &event)
{
    mUI.handleUIPointerDown(event.x, event.y);
}


void TitleState::onMouseUp(const SDL_MouseButtonEvent &event)
{
    mUI.handleUIPointerUp(event.x, event.y);
}


void TitleState::onMouseMotion(const SDL_MouseMotionEvent &event)
{
    mUI.handleUIPointerMove(event.x, event.y);
}


void TitleState::gameStart()
{
    GameApp *app = (GameApp *)App::getInstance();
    if (app->isAlive())
        app->setState(GameApp::STATE_GAME_RUN);
}


void TitleState::onAction(UIComponent *component)
{
    if (component == mPlayButton)
        gameStart();
    else if (component == mExitButton)
        App::getInstance()->setAlive(FALSE);
    playSoundEffect(SOUND_BUTTON);
}
