/* Turska UI library
 * Copyright 2003-2004 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: UIFont.cpp,v 1.2 2006-04-30 14:06:34 tonic Exp $
 * $Revision: 1.2 $
 */

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include "SDL.h"
#include "SDL_image.h"
#include "UI.h"


const static char defaultFontStr[] =
    " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~";


UIFont::UIFont(const char *fileName, const char *fontString) :
    mLoaded(FALSE), mGlyphs(NULL), mGlyphCount(0),
    mCharacterSpacing(1), mFontHeight(0),
    mDrawMode(UIImage::NORMAL)
{
    strncpy(mFileName, fileName, 256);

    if (fontString == NULL)
        fontString = defaultFontStr;
    buildCharMap(fontString);
}


UIFont::~UIFont()
{
    UINT a;
    memset(mFileName, 0, 256);
    memset(mGlyphMap, 0, 256);
    for (a = 0; a < mGlyphCount; ++a)
        delete mGlyphs[a];
    delete[] mGlyphs;
    mGlyphs = NULL;
    mGlyphCount = 0;
    mLoaded = FALSE;
}


void UIFont::buildCharMap(const char *fontString)
{
    UINT a, ch, mapped;

    mGlyphCount = strlen(fontString);

    for (ch = 0; ch < 256; ++ch)
    {
        /* All chars map to first glyph by default
         * if different glyph isn't specified.
         */
        mapped = 0;

        for (a = 0; a < mGlyphCount; ++a)
        {
            if (fontString[a] == (char)ch)
            {
                mapped = a;
                break;
            }
        }

        mGlyphMap[ch] = mapped;
    }
}
